@@ -1,13 +1,11 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
-   +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2014 The PHP Group                                |
+   | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
    | available through the world-wide-web at the following url:           |
-   | http://www.php.net/license/3_01.txt                                  |
+   | https://www.php.net/license/3_01.txt                                 |
    | If you did not receive a copy of the PHP license and are unable to   |
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
@@ -16,13 +14,11 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id$ */
-
 #define _GNU_SOURCE
 #include "php.h"
+#include "ext/standard/base64.h"
 
-PHPAPI int php_url_decode(char *str, int len);
-PHPAPI unsigned char *php_base64_decode(const unsigned char *str, int length, int *ret_length);
+PHPAPI size_t php_url_decode(char *str, size_t len);
 
 /* Memory streams use a dynamic memory buffer to emulate a stream.
  * You can use php_stream_memory_open to create a readonly stream
@@ -37,43 +33,31 @@ PHPAPI unsigned char *php_base64_decode(const unsigned char *str, int length, in
 /* {{{ ------- MEMORY stream implementation -------*/
 
 typedef struct {
-	char        *data;
+	zend_string *data;
 	size_t      fpos;
-	size_t      fsize;
-	size_t      smax;
 	int			mode;
 } php_stream_memory_data;
 
 
 /* {{{ */
-static size_t php_stream_memory_write(php_stream *stream, const char *buf, size_t count TSRMLS_DC)
+static ssize_t php_stream_memory_write(php_stream *stream, const char *buf, size_t count)
 {
 	php_stream_memory_data *ms = (php_stream_memory_data*)stream->abstract;
 	assert(ms != NULL);
 
 	if (ms->mode & TEMP_STREAM_READONLY) {
-		return 0;
+		return (ssize_t) -1;
+	} else if (ms->mode & TEMP_STREAM_APPEND) {
+		ms->fpos = ZSTR_LEN(ms->data);
 	}
-	if (ms->fpos + count > ms->fsize) {
-		char *tmp;
-
-		if (!ms->data) {
-			tmp = emalloc(ms->fpos + count);
-		} else {
-			tmp = erealloc(ms->data, ms->fpos + count);
-		}
-		if (!tmp) {
-			count = ms->fsize - ms->fpos + 1;
-		} else {
-			ms->data = tmp;
-			ms->fsize = ms->fpos + count;
-		}
+	if (ms->fpos + count > ZSTR_LEN(ms->data)) {
+		ms->data = zend_string_realloc(ms->data, ms->fpos + count, 0);
+	} else {
+		ms->data = zend_string_separate(ms->data, 0);
 	}
-	if (!ms->data)
-		count = 0;
 	if (count) {
-		assert(buf!= NULL);
-		memcpy(ms->data+ms->fpos, (char*)buf, count);
+		ZEND_ASSERT(buf != NULL);
+		memcpy(ZSTR_VAL(ms->data) + ms->fpos, (char*) buf, count);
 		ms->fpos += count;
 	}
 	return count;
@@ -82,22 +66,21 @@ static size_t php_stream_memory_write(php_stream *stream, const char *buf, size_
 
 
 /* {{{ */
-static size_t php_stream_memory_read(php_stream *stream, char *buf, size_t count TSRMLS_DC)
+static ssize_t php_stream_memory_read(php_stream *stream, char *buf, size_t count)
 {
 	php_stream_memory_data *ms = (php_stream_memory_data*)stream->abstract;
 	assert(ms != NULL);
 
-	if (ms->fpos == ms->fsize) {
+	if (ms->fpos == ZSTR_LEN(ms->data)) {
 		stream->eof = 1;
 		count = 0;
 	} else {
-		if (ms->fpos + count >= ms->fsize) {
-			count = ms->fsize - ms->fpos;
+		if (ms->fpos + count > ZSTR_LEN(ms->data)) {
+			count = ZSTR_LEN(ms->data) - ms->fpos;
 		}
 		if (count) {
-			assert(ms->data!= NULL);
-			assert(buf!= NULL);
-			memcpy(buf, ms->data+ms->fpos, count);
+			ZEND_ASSERT(buf != NULL);
+			memcpy(buf, ZSTR_VAL(ms->data) + ms->fpos, count);
 			ms->fpos += count;
 		}
 	}
@@ -107,14 +90,11 @@ static size_t php_stream_memory_read(php_stream *stream, char *buf, size_t count
 
 
 /* {{{ */
-static int php_stream_memory_close(php_stream *stream, int close_handle TSRMLS_DC)
+static int php_stream_memory_close(php_stream *stream, int close_handle)
 {
 	php_stream_memory_data *ms = (php_stream_memory_data*)stream->abstract;
-	assert(ms != NULL);
-
-	if (ms->data && close_handle && ms->mode != TEMP_STREAM_READONLY) {
-		efree(ms->data);
-	}
+	ZEND_ASSERT(ms != NULL);
+	zend_string_release(ms->data);
 	efree(ms);
 	return 0;
 }
@@ -122,7 +102,7 @@ static int php_stream_memory_close(php_stream *stream, int close_handle TSRMLS_D
 
 
 /* {{{ */
-static int php_stream_memory_flush(php_stream *stream TSRMLS_DC)
+static int php_stream_memory_flush(php_stream *stream)
 {
 	/* nothing to do here */
 	return 0;
@@ -131,7 +111,7 @@ static int php_stream_memory_flush(php_stream *stream TSRMLS_DC)
 
 
 /* {{{ */
-static int php_stream_memory_seek(php_stream *stream, off_t offset, int whence, off_t *newoffs TSRMLS_DC)
+static int php_stream_memory_seek(php_stream *stream, zend_off_t offset, int whence, zend_off_t *newoffs)
 {
 	php_stream_memory_data *ms = (php_stream_memory_data*)stream->abstract;
 	assert(ms != NULL);
@@ -150,8 +130,8 @@ static int php_stream_memory_seek(php_stream *stream, off_t offset, int whence,
 					return 0;
 				}
 			} else {
-				if (ms->fpos + (size_t)(offset) > ms->fsize) {
-					ms->fpos = ms->fsize;
+				if (ms->fpos + (size_t)(offset) > ZSTR_LEN(ms->data)) {
+					ms->fpos = ZSTR_LEN(ms->data);
 					*newoffs = -1;
 					return -1;
 				} else {
@@ -162,8 +142,8 @@ static int php_stream_memory_seek(php_stream *stream, off_t offset, int whence,
 				}
 			}
 		case SEEK_SET:
-			if (ms->fsize < (size_t)(offset)) {
-				ms->fpos = ms->fsize;
+			if (ZSTR_LEN(ms->data) < (size_t)(offset)) {
+				ms->fpos = ZSTR_LEN(ms->data);
 				*newoffs = -1;
 				return -1;
 			} else {
@@ -174,15 +154,15 @@ static int php_stream_memory_seek(php_stream *stream, off_t offset, int whence,
 			}
 		case SEEK_END:
 			if (offset > 0) {
-				ms->fpos = ms->fsize;
+				ms->fpos = ZSTR_LEN(ms->data);
 				*newoffs = -1;
 				return -1;
-			} else if (ms->fsize < (size_t)(-offset)) {
+			} else if (ZSTR_LEN(ms->data) < (size_t)(-offset)) {
 				ms->fpos = 0;
 				*newoffs = -1;
 				return -1;
 			} else {
-				ms->fpos = ms->fsize + offset;
+				ms->fpos = ZSTR_LEN(ms->data) + offset;
 				*newoffs = ms->fpos;
 				stream->eof = 0;
 				return 0;
@@ -195,13 +175,13 @@ static int php_stream_memory_seek(php_stream *stream, off_t offset, int whence,
 /* }}} */
 
 /* {{{ */
-static int php_stream_memory_cast(php_stream *stream, int castas, void **ret TSRMLS_DC)
+static int php_stream_memory_cast(php_stream *stream, int castas, void **ret)
 {
 	return FAILURE;
 }
 /* }}} */
 
-static int php_stream_memory_stat(php_stream *stream, php_stream_statbuf *ssb TSRMLS_DC) /* {{{ */
+static int php_stream_memory_stat(php_stream *stream, php_stream_statbuf *ssb) /* {{{ */
 {
 	time_t timestamp = 0;
 	php_stream_memory_data *ms = (php_stream_memory_data*)stream->abstract;
@@ -209,22 +189,14 @@ static int php_stream_memory_stat(php_stream *stream, php_stream_statbuf *ssb TS
 
 	memset(ssb, 0, sizeof(php_stream_statbuf));
 	/* read-only across the board */
-	
+
 	ssb->sb.st_mode = ms->mode & TEMP_STREAM_READONLY ? 0444 : 0666;
 
-	ssb->sb.st_size = ms->fsize;
+	ssb->sb.st_size = ZSTR_LEN(ms->data);
 	ssb->sb.st_mode |= S_IFREG; /* regular file */
-
-#ifdef NETWARE
-	ssb->sb.st_mtime.tv_sec = timestamp;
-	ssb->sb.st_atime.tv_sec = timestamp;
-	ssb->sb.st_ctime.tv_sec = timestamp;
-#else
 	ssb->sb.st_mtime = timestamp;
 	ssb->sb.st_atime = timestamp;
 	ssb->sb.st_ctime = timestamp;
-#endif
-
 	ssb->sb.st_nlink = 1;
 	ssb->sb.st_rdev = -1;
 	/* this is only for APC, so use /dev/null device - no chance of conflict there! */
@@ -234,9 +206,6 @@ static int php_stream_memory_stat(php_stream *stream, php_stream_statbuf *ssb TS
 
 #ifndef PHP_WIN32
 	ssb->sb.st_blksize = -1;
-#endif
-
-#if !defined(PHP_WIN32) && !defined(__BEOS__)
 	ssb->sb.st_blocks = -1;
 #endif
 
@@ -244,11 +213,11 @@ static int php_stream_memory_stat(php_stream *stream, php_stream_statbuf *ssb TS
 }
 /* }}} */
 
-static int php_stream_memory_set_option(php_stream *stream, int option, int value, void *ptrparam TSRMLS_DC) /* {{{ */
+static int php_stream_memory_set_option(php_stream *stream, int option, int value, void *ptrparam) /* {{{ */
 {
 	php_stream_memory_data *ms = (php_stream_memory_data*)stream->abstract;
 	size_t newsize;
-	
+
 	switch(option) {
 		case PHP_STREAM_OPTION_TRUNCATE_API:
 			switch (value) {
@@ -260,25 +229,25 @@ static int php_stream_memory_set_option(php_stream *stream, int option, int valu
 						return PHP_STREAM_OPTION_RETURN_ERR;
 					}
 					newsize = *(size_t*)ptrparam;
-					if (newsize <= ms->fsize) {
+					if (newsize <= ZSTR_LEN(ms->data)) {
+						ms->data = zend_string_truncate(ms->data, newsize, 0);
 						if (newsize < ms->fpos) {
 							ms->fpos = newsize;
 						}
 					} else {
-						ms->data = erealloc(ms->data, newsize);
-						memset(ms->data+ms->fsize, 0, newsize - ms->fsize);
-						ms->fsize = newsize;
+						size_t old_size = ZSTR_LEN(ms->data);
+						ms->data = zend_string_realloc(ms->data, newsize, 0);
+						memset(ZSTR_VAL(ms->data) + old_size, 0, newsize - old_size);
 					}
-					ms->fsize = newsize;
 					return PHP_STREAM_OPTION_RETURN_OK;
 			}
-		default:
-			return PHP_STREAM_OPTION_RETURN_NOTIMPL;
 	}
+
+	return PHP_STREAM_OPTION_RETURN_NOTIMPL;
 }
 /* }}} */
-	
-PHPAPI php_stream_ops	php_stream_memory_ops = {
+
+PHPAPI const php_stream_ops	php_stream_memory_ops = {
 	php_stream_memory_write, php_stream_memory_read,
 	php_stream_memory_close, php_stream_memory_flush,
 	"MEMORY",
@@ -288,21 +257,42 @@ PHPAPI php_stream_ops	php_stream_memory_ops = {
 	php_stream_memory_set_option
 };
 
+/* {{{ */
+PHPAPI int php_stream_mode_from_str(const char *mode)
+{
+	if (strpbrk(mode, "a")) {
+		return TEMP_STREAM_APPEND;
+	} else if (strpbrk(mode, "w+")) {
+		return TEMP_STREAM_DEFAULT;
+	}
+	return TEMP_STREAM_READONLY;
+}
+/* }}} */
 
 /* {{{ */
-PHPAPI php_stream *_php_stream_memory_create(int mode STREAMS_DC TSRMLS_DC)
+PHPAPI const char *_php_stream_mode_to_str(int mode)
+{
+	if (mode == TEMP_STREAM_READONLY) {
+		return "rb";
+	} else if (mode == TEMP_STREAM_APPEND) {
+		return "a+b";
+	}
+	return "w+b";
+}
+/* }}} */
+
+/* {{{ */
+PHPAPI php_stream *_php_stream_memory_create(int mode STREAMS_DC)
 {
 	php_stream_memory_data *self;
 	php_stream *stream;
 
 	self = emalloc(sizeof(*self));
-	self->data = NULL;
+	self->data = ZSTR_EMPTY_ALLOC();
 	self->fpos = 0;
-	self->fsize = 0;
-	self->smax = ~0u;
 	self->mode = mode;
-	
-	stream = php_stream_alloc_rel(&php_stream_memory_ops, self, 0, mode & TEMP_STREAM_READONLY ? "rb" : "w+b");
+
+	stream = php_stream_alloc_rel(&php_stream_memory_ops, self, 0, _php_stream_mode_to_str(mode));
 	stream->flags |= PHP_STREAM_FLAG_NO_BUFFER;
 	return stream;
 }
@@ -310,24 +300,14 @@ PHPAPI php_stream *_php_stream_memory_create(int mode STREAMS_DC TSRMLS_DC)
 
 
 /* {{{ */
-PHPAPI php_stream *_php_stream_memory_open(int mode, char *buf, size_t length STREAMS_DC TSRMLS_DC)
+PHPAPI php_stream *_php_stream_memory_open(int mode, zend_string *buf STREAMS_DC)
 {
 	php_stream *stream;
 	php_stream_memory_data *ms;
 
 	if ((stream = php_stream_memory_create_rel(mode)) != NULL) {
 		ms = (php_stream_memory_data*)stream->abstract;
-		
-		if (mode == TEMP_STREAM_READONLY || mode == TEMP_STREAM_TAKE_BUFFER) {
-			/* use the buffer directly */
-			ms->data = buf;
-			ms->fsize = length;
-		} else {
-			if (length) {
-				assert(buf != NULL);
-				php_stream_write(stream, buf, length);
-			}
-		}
+		ms->data = zend_string_copy(buf);
 	}
 	return stream;
 }
@@ -335,14 +315,10 @@ PHPAPI php_stream *_php_stream_memory_open(int mode, char *buf, size_t length ST
 
 
 /* {{{ */
-PHPAPI char *_php_stream_memory_get_buffer(php_stream *stream, size_t *length STREAMS_DC TSRMLS_DC)
+PHPAPI zend_string *_php_stream_memory_get_buffer(php_stream *stream STREAMS_DC)
 {
 	php_stream_memory_data *ms = (php_stream_memory_data*)stream->abstract;
-
-	assert(ms != NULL);
-	assert(length != 0);
-
-	*length = ms->fsize;
+	ZEND_ASSERT(ms != NULL);
 	return ms->data;
 }
 /* }}} */
@@ -355,13 +331,13 @@ typedef struct {
 	php_stream  *innerstream;
 	size_t      smax;
 	int			mode;
-	zval*       meta;
+	zval        meta;
 	char*		tmpdir;
 } php_stream_temp_data;
 
 
 /* {{{ */
-static size_t php_stream_temp_write(php_stream *stream, const char *buf, size_t count TSRMLS_DC)
+static ssize_t php_stream_temp_write(php_stream *stream, const char *buf, size_t count)
 {
 	php_stream_temp_data *ts = (php_stream_temp_data*)stream->abstract;
 	assert(ts != NULL);
@@ -370,12 +346,15 @@ static size_t php_stream_temp_write(php_stream *stream, const char *buf, size_t
 		return -1;
 	}
 	if (php_stream_is(ts->innerstream, PHP_STREAM_IS_MEMORY)) {
-		size_t memsize;
-		char *membuf = php_stream_memory_get_buffer(ts->innerstream, &memsize);
+		zend_string *membuf = php_stream_memory_get_buffer(ts->innerstream);
 
-		if (memsize + count >= ts->smax) {
+		if (ZSTR_LEN(membuf) + count >= ts->smax) {
 			php_stream *file = php_stream_fopen_temporary_file(ts->tmpdir, "php", NULL);
-			php_stream_write(file, membuf, memsize);
+			if (file == NULL) {
+				php_error_docref(NULL, E_WARNING, "Unable to create temporary file, Check permissions in temporary files directory.");
+				return 0;
+			}
+			php_stream_write(file, ZSTR_VAL(membuf), ZSTR_LEN(membuf));
 			php_stream_free_enclosed(ts->innerstream, PHP_STREAM_FREE_CLOSE);
 			ts->innerstream = file;
 			php_stream_encloses(stream, ts->innerstream);
@@ -387,7 +366,7 @@ static size_t php_stream_temp_write(php_stream *stream, const char *buf, size_t
 
 
 /* {{{ */
-static size_t php_stream_temp_read(php_stream *stream, char *buf, size_t count TSRMLS_DC)
+static ssize_t php_stream_temp_read(php_stream *stream, char *buf, size_t count)
 {
 	php_stream_temp_data *ts = (php_stream_temp_data*)stream->abstract;
 	size_t got;
@@ -397,18 +376,18 @@ static size_t php_stream_temp_read(php_stream *stream, char *buf, size_t count T
 	if (!ts->innerstream) {
 		return -1;
 	}
-	
+
 	got = php_stream_read(ts->innerstream, buf, count);
-	
+
 	stream->eof = ts->innerstream->eof;
-	
+
 	return got;
 }
 /* }}} */
 
 
 /* {{{ */
-static int php_stream_temp_close(php_stream *stream, int close_handle TSRMLS_DC)
+static int php_stream_temp_close(php_stream *stream, int close_handle)
 {
 	php_stream_temp_data *ts = (php_stream_temp_data*)stream->abstract;
 	int ret;
@@ -420,10 +399,8 @@ static int php_stream_temp_close(php_stream *stream, int close_handle TSRMLS_DC)
 	} else {
 		ret = 0;
 	}
-	
-	if (ts->meta) {
-		zval_ptr_dtor(&ts->meta);
-	}
+
+	zval_ptr_dtor(&ts->meta);
 
 	if (ts->tmpdir) {
 		efree(ts->tmpdir);
@@ -437,7 +414,7 @@ static int php_stream_temp_close(php_stream *stream, int close_handle TSRMLS_DC)
 
 
 /* {{{ */
-static int php_stream_temp_flush(php_stream *stream TSRMLS_DC)
+static int php_stream_temp_flush(php_stream *stream)
 {
 	php_stream_temp_data *ts = (php_stream_temp_data*)stream->abstract;
 	assert(ts != NULL);
@@ -448,7 +425,7 @@ static int php_stream_temp_flush(php_stream *stream TSRMLS_DC)
 
 
 /* {{{ */
-static int php_stream_temp_seek(php_stream *stream, off_t offset, int whence, off_t *newoffs TSRMLS_DC)
+static int php_stream_temp_seek(php_stream *stream, zend_off_t offset, int whence, zend_off_t *newoffs)
 {
 	php_stream_temp_data *ts = (php_stream_temp_data*)stream->abstract;
 	int ret;
@@ -462,19 +439,18 @@ static int php_stream_temp_seek(php_stream *stream, off_t offset, int whence, of
 	ret = php_stream_seek(ts->innerstream, offset, whence);
 	*newoffs = php_stream_tell(ts->innerstream);
 	stream->eof = ts->innerstream->eof;
-	
+
 	return ret;
 }
 /* }}} */
 
 /* {{{ */
-static int php_stream_temp_cast(php_stream *stream, int castas, void **ret TSRMLS_DC)
+static int php_stream_temp_cast(php_stream *stream, int castas, void **ret)
 {
 	php_stream_temp_data *ts = (php_stream_temp_data*)stream->abstract;
 	php_stream *file;
-	size_t memsize;
-	char *membuf;
-	off_t pos;
+	zend_string *membuf;
+	zend_off_t pos;
 
 	assert(ts != NULL);
 
@@ -499,12 +475,17 @@ static int php_stream_temp_cast(php_stream *stream, int castas, void **ret TSRML
 		return FAILURE;
 	}
 
-	/* perform the conversion and then pass the request on to the innerstream */
-	membuf = php_stream_memory_get_buffer(ts->innerstream, &memsize);
 	file = php_stream_fopen_tmpfile();
-	php_stream_write(file, membuf, memsize);
+	if (file == NULL) {
+		php_error_docref(NULL, E_WARNING, "Unable to create temporary file.");
+		return FAILURE;
+	}
+
+	/* perform the conversion and then pass the request on to the innerstream */
+	membuf = php_stream_memory_get_buffer(ts->innerstream);
+	php_stream_write(file, ZSTR_VAL(membuf), ZSTR_LEN(membuf));
 	pos = php_stream_tell(ts->innerstream);
-	
+
 	php_stream_free_enclosed(ts->innerstream, PHP_STREAM_FREE_CLOSE);
 	ts->innerstream = file;
 	php_stream_encloses(stream, ts->innerstream);
@@ -514,7 +495,7 @@ static int php_stream_temp_cast(php_stream *stream, int castas, void **ret TSRML
 }
 /* }}} */
 
-static int php_stream_temp_stat(php_stream *stream, php_stream_statbuf *ssb TSRMLS_DC) /* {{{ */
+static int php_stream_temp_stat(php_stream *stream, php_stream_statbuf *ssb) /* {{{ */
 {
 	php_stream_temp_data *ts = (php_stream_temp_data*)stream->abstract;
 
@@ -525,14 +506,14 @@ static int php_stream_temp_stat(php_stream *stream, php_stream_statbuf *ssb TSRM
 }
 /* }}} */
 
-static int php_stream_temp_set_option(php_stream *stream, int option, int value, void *ptrparam TSRMLS_DC) /* {{{ */
+static int php_stream_temp_set_option(php_stream *stream, int option, int value, void *ptrparam) /* {{{ */
 {
 	php_stream_temp_data *ts = (php_stream_temp_data*)stream->abstract;
-	
+
 	switch(option) {
 		case PHP_STREAM_OPTION_META_DATA_API:
-			if (ts->meta) {
-				zend_hash_copy(Z_ARRVAL_P((zval*)ptrparam), Z_ARRVAL_P(ts->meta), (copy_ctor_func_t) zval_add_ref, NULL, sizeof(zval*));
+			if (Z_TYPE(ts->meta) != IS_UNDEF) {
+				zend_hash_copy(Z_ARRVAL_P((zval*)ptrparam), Z_ARRVAL(ts->meta), zval_add_ref);
 			}
 			return PHP_STREAM_OPTION_RETURN_OK;
 		default:
@@ -544,7 +525,7 @@ static int php_stream_temp_set_option(php_stream *stream, int option, int value,
 }
 /* }}} */
 
-PHPAPI php_stream_ops	php_stream_temp_ops = {
+PHPAPI const php_stream_ops	php_stream_temp_ops = {
 	php_stream_temp_write, php_stream_temp_read,
 	php_stream_temp_close, php_stream_temp_flush,
 	"TEMP",
@@ -557,7 +538,7 @@ PHPAPI php_stream_ops	php_stream_temp_ops = {
 /* }}} */
 
 /* {{{ _php_stream_temp_create_ex */
-PHPAPI php_stream *_php_stream_temp_create_ex(int mode, size_t max_memory_usage, const char *tmpdir STREAMS_DC TSRMLS_DC)
+PHPAPI php_stream *_php_stream_temp_create_ex(int mode, size_t max_memory_usage, const char *tmpdir STREAMS_DC)
 {
 	php_stream_temp_data *self;
 	php_stream *stream;
@@ -565,10 +546,11 @@ PHPAPI php_stream *_php_stream_temp_create_ex(int mode, size_t max_memory_usage,
 	self = ecalloc(1, sizeof(*self));
 	self->smax = max_memory_usage;
 	self->mode = mode;
+	ZVAL_UNDEF(&self->meta);
 	if (tmpdir) {
 		self->tmpdir = estrdup(tmpdir);
 	}
-	stream = php_stream_alloc_rel(&php_stream_temp_ops, self, 0, mode & TEMP_STREAM_READONLY ? "rb" : "w+b");
+	stream = php_stream_alloc_rel(&php_stream_temp_ops, self, 0, _php_stream_mode_to_str(mode));
 	stream->flags |= PHP_STREAM_FLAG_NO_BUFFER;
 	self->innerstream = php_stream_memory_create_rel(mode);
 	php_stream_encloses(stream, self->innerstream);
@@ -578,24 +560,24 @@ PHPAPI php_stream *_php_stream_temp_create_ex(int mode, size_t max_memory_usage,
 /* }}} */
 
 /* {{{ _php_stream_temp_create */
-PHPAPI php_stream *_php_stream_temp_create(int mode, size_t max_memory_usage STREAMS_DC TSRMLS_DC)
+PHPAPI php_stream *_php_stream_temp_create(int mode, size_t max_memory_usage STREAMS_DC)
 {
 	return php_stream_temp_create_ex(mode, max_memory_usage, NULL);
 }
 /* }}} */
 
 /* {{{ _php_stream_temp_open */
-PHPAPI php_stream *_php_stream_temp_open(int mode, size_t max_memory_usage, char *buf, size_t length STREAMS_DC TSRMLS_DC)
+PHPAPI php_stream *_php_stream_temp_open(int mode, size_t max_memory_usage, const char *buf, size_t length STREAMS_DC)
 {
 	php_stream *stream;
 	php_stream_temp_data *ts;
-	off_t newoffs;
+	zend_off_t newoffs;
 
 	if ((stream = php_stream_temp_create_rel(mode, max_memory_usage)) != NULL) {
 		if (length) {
 			assert(buf != NULL);
-			php_stream_temp_write(stream, buf, length TSRMLS_CC);
-			php_stream_temp_seek(stream, 0, SEEK_SET, &newoffs TSRMLS_CC);
+			php_stream_temp_write(stream, buf, length);
+			php_stream_temp_seek(stream, 0, SEEK_SET, &newoffs);
 		}
 		ts = (php_stream_temp_data*)stream->abstract;
 		assert(ts != NULL);
@@ -605,8 +587,8 @@ PHPAPI php_stream *_php_stream_temp_open(int mode, size_t max_memory_usage, char
 }
 /* }}} */
 
-PHPAPI php_stream_ops php_stream_rfc2397_ops = {
-	php_stream_temp_write, php_stream_temp_read,
+PHPAPI const php_stream_ops php_stream_rfc2397_ops = {
+	NULL, php_stream_temp_read,
 	php_stream_temp_close, php_stream_temp_flush,
 	"RFC2397",
 	php_stream_temp_seek,
@@ -616,17 +598,19 @@ PHPAPI php_stream_ops php_stream_rfc2397_ops = {
 };
 
 static php_stream * php_stream_url_wrap_rfc2397(php_stream_wrapper *wrapper, const char *path,
-												const char *mode, int options, char **opened_path,
-												php_stream_context *context STREAMS_DC TSRMLS_DC) /* {{{ */
+												const char *mode, int options, zend_string **opened_path,
+												php_stream_context *context STREAMS_DC) /* {{{ */
 {
 	php_stream *stream;
 	php_stream_temp_data *ts;
-	char *comma, *semi, *sep, *key;
-	size_t mlen, dlen, plen, vlen;
-	off_t newoffs;
-	zval *meta = NULL;
-	int base64 = 0, ilen;
+	char *comma, *semi, *sep;
+	size_t mlen, dlen, plen, vlen, ilen;
+	zend_off_t newoffs;
+	zval meta;
+	int base64 = 0;
+	zend_string *base64_comma = NULL;
 
+	ZVAL_NULL(&meta);
 	if (memcmp(path, "data:", 5)) {
 		return NULL;
 	}
@@ -640,7 +624,7 @@ static php_stream * php_stream_url_wrap_rfc2397(php_stream_wrapper *wrapper, con
 	}
 
 	if ((comma = memchr(path, ',', dlen)) == NULL) {
-		php_stream_wrapper_log_error(wrapper, options TSRMLS_CC, "rfc2397: no comma in URL");
+		php_stream_wrapper_log_error(wrapper, options, "rfc2397: no comma in URL");
 		return NULL;
 	}
 
@@ -650,25 +634,24 @@ static php_stream * php_stream_url_wrap_rfc2397(php_stream_wrapper *wrapper, con
 		dlen -= mlen;
 		semi = memchr(path, ';', mlen);
 		sep = memchr(path, '/', mlen);
-		
+
 		if (!semi && !sep) {
-			php_stream_wrapper_log_error(wrapper, options TSRMLS_CC, "rfc2397: illegal media type");
+			php_stream_wrapper_log_error(wrapper, options, "rfc2397: illegal media type");
 			return NULL;
 		}
 
-		MAKE_STD_ZVAL(meta);
-		array_init(meta);
+		array_init(&meta);
 		if (!semi) { /* there is only a mime type */
-			add_assoc_stringl(meta, "mediatype", (char *) path, mlen, 1);
+			add_assoc_stringl(&meta, "mediatype", (char *) path, mlen);
 			mlen = 0;
 		} else if (sep && sep < semi) { /* there is a mime type */
 			plen = semi - path;
-			add_assoc_stringl(meta, "mediatype", (char *) path, plen, 1);
+			add_assoc_stringl(&meta, "mediatype", (char *) path, plen);
 			mlen -= plen;
 			path += plen;
 		} else if (semi != path || mlen != sizeof(";base64")-1 || memcmp(path, ";base64", sizeof(";base64")-1)) { /* must be error since parameters are only allowed after mediatype */
 			zval_ptr_dtor(&meta);
-			php_stream_wrapper_log_error(wrapper, options TSRMLS_CC, "rfc2397: illegal media type");
+			php_stream_wrapper_log_error(wrapper, options, "rfc2397: illegal media type");
 			return NULL;
 		}
 		/* get parameters and potentially ';base64' */
@@ -681,7 +664,7 @@ static php_stream * php_stream_url_wrap_rfc2397(php_stream_wrapper *wrapper, con
 				if (mlen != sizeof("base64")-1 || memcmp(path, "base64", sizeof("base64")-1)) {
 					/* must be error since parameters are only allowed after mediatype and we have no '=' sign */
 					zval_ptr_dtor(&meta);
-					php_stream_wrapper_log_error(wrapper, options TSRMLS_CC, "rfc2397: illegal parameter");
+					php_stream_wrapper_log_error(wrapper, options, "rfc2397: illegal parameter");
 					return NULL;
 				}
 				base64 = 1;
@@ -691,45 +674,47 @@ static php_stream * php_stream_url_wrap_rfc2397(php_stream_wrapper *wrapper, con
 			}
 			/* found parameter ... the heart of cs ppl lies in +1/-1 or was it +2 this time? */
 			plen = sep - path;
-			vlen = (semi ? semi - sep : mlen - plen) - 1 /* '=' */;
-			key = estrndup(path, plen);
-			add_assoc_stringl_ex(meta, key, plen + 1, sep + 1, vlen, 1);
-			efree(key);
+			vlen = (semi ? (size_t)(semi - sep) : (mlen - plen)) - 1 /* '=' */;
+			if (plen != sizeof("mediatype")-1 || memcmp(path, "mediatype", sizeof("mediatype")-1)) {
+				add_assoc_stringl_ex(&meta, path, plen, sep + 1, vlen);
+			}
 			plen += vlen + 1;
 			mlen -= plen;
 			path += plen;
 		}
 		if (mlen) {
 			zval_ptr_dtor(&meta);
-			php_stream_wrapper_log_error(wrapper, options TSRMLS_CC, "rfc2397: illegal URL");
+			php_stream_wrapper_log_error(wrapper, options, "rfc2397: illegal URL");
 			return NULL;
 		}
 	} else {
-		MAKE_STD_ZVAL(meta);
-		array_init(meta);
+		array_init(&meta);
 	}
-	add_assoc_bool(meta, "base64", base64);
+	add_assoc_bool(&meta, "base64", base64);
 
 	/* skip ',' */
 	comma++;
 	dlen--;
 
 	if (base64) {
-		comma = (char*)php_base64_decode((const unsigned char *)comma, dlen, &ilen);
-		if (!comma) {
+		base64_comma = php_base64_decode_ex((const unsigned char *)comma, dlen, 1);
+		if (!base64_comma) {
 			zval_ptr_dtor(&meta);
-			php_stream_wrapper_log_error(wrapper, options TSRMLS_CC, "rfc2397: unable to decode");
+			php_stream_wrapper_log_error(wrapper, options, "rfc2397: unable to decode");
 			return NULL;
 		}
+		comma = ZSTR_VAL(base64_comma);
+		ilen = ZSTR_LEN(base64_comma);
 	} else {
 		comma = estrndup(comma, dlen);
-		ilen = dlen = php_url_decode(comma, dlen);
+		dlen = php_url_decode(comma, dlen);
+		ilen = dlen;
 	}
 
 	if ((stream = php_stream_temp_create_rel(0, ~0u)) != NULL) {
 		/* store data */
-		php_stream_temp_write(stream, comma, ilen TSRMLS_CC);
-		php_stream_temp_seek(stream, 0, SEEK_SET, &newoffs TSRMLS_CC);
+		php_stream_temp_write(stream, comma, ilen);
+		php_stream_temp_seek(stream, 0, SEEK_SET, &newoffs);
 		/* set special stream stuff (enforce exact mode) */
 		vlen = strlen(mode);
 		if (vlen >= sizeof(stream->mode)) {
@@ -741,14 +726,18 @@ static php_stream * php_stream_url_wrap_rfc2397(php_stream_wrapper *wrapper, con
 		ts = (php_stream_temp_data*)stream->abstract;
 		assert(ts != NULL);
 		ts->mode = mode && mode[0] == 'r' && mode[1] != '+' ? TEMP_STREAM_READONLY : 0;
-		ts->meta = meta;
+		ZVAL_COPY_VALUE(&ts->meta, &meta);
+	}
+	if (base64_comma) {
+		zend_string_free(base64_comma);
+	} else {
+		efree(comma);
 	}
-	efree(comma);
 
 	return stream;
 }
 
-PHPAPI php_stream_wrapper_ops php_stream_rfc2397_wops = {
+PHPAPI const php_stream_wrapper_ops php_stream_rfc2397_wops = {
 	php_stream_url_wrap_rfc2397,
 	NULL, /* close */
 	NULL, /* fstat */
@@ -758,20 +747,12 @@ PHPAPI php_stream_wrapper_ops php_stream_rfc2397_wops = {
 	NULL, /* unlink */
 	NULL, /* rename */
 	NULL, /* mkdir */
-	NULL  /* rmdir */
+	NULL, /* rmdir */
+	NULL, /* stream_metadata */
 };
 
-PHPAPI php_stream_wrapper php_stream_rfc2397_wrapper =	{
+PHPAPI const php_stream_wrapper php_stream_rfc2397_wrapper =	{
 	&php_stream_rfc2397_wops,
 	NULL,
 	1, /* is_url */
 };
-
-/*
- * Local variables:
- * tab-width: 4
- * c-basic-offset: 4
- * End:
- * vim600: noet sw=4 ts=4 fdm=marker
- * vim<600: noet sw=4 ts=4
- */
